==> funcs_aux-0.3.13-py3-none-any.whl: funcs_aux/result_cum.py <==
from typing import *
from funcs_aux import *
from object_info import *


# =====================================================================================================================
TYPE__RESULT_CUM_STEP = Union[bool, Any, ValueValidate]     # DONT USE CALLABLES!!! ValueValidate is most PREFIRED! specially in case of EXX!!
TYPE__RESULT_CUM_STEPS = Union[TYPE__RESULT_CUM_STEP, list[TYPE__RESULT_CUM_STEP]]


# =====================================================================================================================
class ResultCum:
    """
    GOAL
    ----
    apply cumulated result with log lines

    CREATED SPECIALLY FOR
    ---------------------
    testplans

    REPLACE: ResultExpected

    CONSTRAINTS
    -----------
    1. dont use callables in steps! only final result (as expression) or ValueValidate as most preferred
    2. if no steps - return True!

    BEST USAGE
    ----------
        # some example
        def meth() -> ResultCum:
            result_cum = ResultCum()

            result_step = 1+1 == 2
            result_cum.result__apply_step(result_step)
            result_cum.log_lines__add('check 1+1 == 2')

            result_step = 1+2 == 4
            result_cum.result__apply_step(result_step)
            result_cum.log_lines__add('check 1+2 == 4')

            result_cum.finish()
            return result_cum

    """

    TITLE: str = ""
    # COMMENT: str = ""

    result: bool = True
    finished: bool | None = None    # as help to see if process is finished - maybe need deprecate!

    LOG_LINES: list[str]
    STEP_HISTORY: list[tuple[TYPE__RESULT_CUM_STEP, bool]]     # as history! result+step

    def __init__(self, title: str = None):        #, steps=None):  #dont use steps here
        if title:
            self.TITLE = title

        self.clear()

    def clear(self) -> None:
        self.result = True
        self.finished = None

        self.LOG_LINES = []
        self.STEP_HISTORY = []

    def result__apply_step(self, step: TYPE__RESULT_CUM_STEPS, cumulate: bool | Any = True, msg: Union[None, str, list[str]] = None) -> bool:
        """

        :param step:
        :param cumulate: False - if you dont mind step result! but want to keep it in log
        :param msg:
        :return:
        """
        if isinstance(step, list):
            # LIST ------------------------------------------
            for step_i in step:
                if not self.result__apply_step(step_i, cumulate):
                    break
        else:
            # SINGLE ------------------------------------------
            # TODO: if isinstance(step, ResultCum):   - what do i need to do here?? decide!
            if isinstance(step, ValueValidate):
                if not step.finished:
                    step.run()
                self.log_lines__add(step.log_last)

        if msg:
            self.log_lines__add(msg)

        step_result = bool(step)
        self.STEP_HISTORY.append((step_result, step))

        if bool(cumulate):
            if self.result is None:
                self.result = step_result
            else:
                self.result &= step_result

        return self.result

    def log_lines__add(self, new: Union[str, list[str]]) -> None:
        if isinstance(new, list):
            self.LOG_LINES.extend(new)
        if isinstance(new, str):
            self.LOG_LINES.append(new)

    def finish(self) -> None:
        self.finished = True

    def __bool__(self) -> bool:
        return bool(self.result)

    def __str__(self):
        result = f"{self.__class__.__name__}(result={self.result},finished={self.finished},TITLE={self.TITLE},LOG_LINES={self.LOG_LINES})"
        return result

    def __repr__(self):
        return str(self)

    def step_last__get(self) -> Any | NoReturn:
        return self.STEP_HISTORY[-1][1]

    def log_last__get(self) -> str | None:
        try:
            return self.LOG_LINES[-1]
        except:
            pass


# =====================================================================================================================

==> funcs_aux-0.3.14-py3-none-any.whl: funcs_aux/result_cum.py <==
from typing import *
from funcs_aux import *
from object_info import *


# =====================================================================================================================
TYPE__RESULT_CUM_STEP = Union[bool, Any, ValueValidate]     # DONT USE CALLABLES!!! ValueValidate is most PREFIRED! specially in case of EXX!!
TYPE__RESULT_CUM_STEPS = Union[TYPE__RESULT_CUM_STEP, list[TYPE__RESULT_CUM_STEP]]


# =====================================================================================================================
class ResultCum:
    """
    GOAL
    ----
    apply cumulated result with log lines

    CREATED SPECIALLY FOR
    ---------------------
    testplans

    REPLACE: ResultExpected

    CONSTRAINTS
    -----------
    1. dont use callables in steps! only final result (as expression) or ValueValidate as most preferred
    2. if no steps - return True!

    BEST USAGE
    ----------
        # some example
        def meth() -> ResultCum:
            result_cum = ResultCum()

            result_step = 1+1 == 2
            result_cum.result__apply_step(result_step)
            result_cum.log_lines__add('check 1+1 == 2')

            result_step = 1+2 == 4
            result_cum.result__apply_step(result_step)
            result_cum.log_lines__add('check 1+2 == 4')

            result_cum.finish()
            return result_cum

    """

    TITLE: str = ""
    # COMMENT: str = ""

    result: bool = True
    finished: bool | None = None    # as help to see if process is finished - maybe need deprecate!

    LOG_LINES: list[str]
    STEP_HISTORY: list[tuple[TYPE__RESULT_CUM_STEP, bool]]     # as history! result+step

    def __init__(self, title: str = None):        #, steps=None):  #dont use steps here
        if title:
            self.TITLE = title

        self.clear()

    def clear(self) -> None:
        self.result = True
        self.finished = None

        self.LOG_LINES = []
        self.STEP_HISTORY = []

    def result__apply_step(self, step: TYPE__RESULT_CUM_STEPS, cumulate: bool | Any = True, msg: Union[None, str, list[str]] = None) -> bool:
        """

        :param step:
        :param cumulate: False - if you dont mind step result! but want to keep it in log
        :param msg:
        :return:
        """
        if isinstance(step, list):
            # LIST ------------------------------------------
            for step_i in step:
                if not self.result__apply_step(step_i, cumulate):
                    break
        else:
            # SINGLE ------------------------------------------
            # TODO: if isinstance(step, ResultCum):   - what do i need to do here?? decide!
            if isinstance(step, ValueValidate):
                if not step.finished:
                    step.run()
                self.log_lines__add(step.log_last)

        if msg:
            self.log_lines__add(msg)

        step_result = bool(step)
        self.STEP_HISTORY.append((step_result, step))

        if bool(cumulate):
            if self.result is None:
                self.result = step_result
            else:
                self.result &= step_result

        return self.result

    def log_lines__add(self, new: Union[str, list[str]]) -> None:
        if isinstance(new, list):
            self.LOG_LINES.extend(new)
        if isinstance(new, str):
            self.LOG_LINES.append(new)

    def finish(self) -> None:
        self.finished = True

    def __bool__(self) -> bool:
        return bool(self.result)

    def __str__(self):
        result = f"{self.__class__.__name__}(result={self.result},finished={self.finished},TITLE={self.TITLE},LOG_LINES:"
        for i, line in enumerate(self.LOG_LINES):
            result += f"\n{i:->5d}:{line}"
        return result

    def __repr__(self):
        return str(self)

    def step_last__get(self) -> Any | NoReturn:
        return self.STEP_HISTORY[-1][1]

    def log_last__get(self) -> str | None:
        try:
            return self.LOG_LINES[-1]
        except:
            pass


# =====================================================================================================================
